#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <winnt.h>

int enc1( FILE *fp, short *buf, int size);

typedef struct {
	int RiffHeader;
	unsigned int FileSize;
	int WavHeader;
} WavFileHead;
WavFileHead stFileHead = { 'FFIR', 0, 'EVAW'};

typedef struct {
  int  chunkID;
  long           chunkSize; 
  short          wFormatTag;
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;
} FormatChunk;
FormatChunk stWavFormat = { ' tmf', 0x10, (short)1, 2, 48000, 48000*2*2, 2*2, 16};

typedef struct {
  int  chunkID;
  long           chunkSize; 
  unsigned char  waveformData[];
} DataChunk;
DataChunk stWavData = { 'atad', 0};

unsigned char pv3head[0x400] = 
{ 0x70, 0x76, 0x33, 0x73, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x01, 0x02, 0x01, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00};

unsigned char nullbuff[0x800];

void main( int argc, char* argv[])
{
	FILE *fp, *fp2;
	int bytes, i, wavs;
	char *buf;
	extern errno;
	int datasize;

	if( argc < 3){
		printf("PV3WAV  PV3.DAT *.WAV\n");
		exit( 1);
	}

	if ((fp = fopen(argv[ 1], "wb")) == NULL){
		printf("fopen error %d\n", errno);
		exit(1);
	}

	for( i=0x18; i<0x400; i++)
		pv3head[ i] = 0x00;
	pv3head[ 0x3Fc] = 0x01;
	for( i=0; i<0x800; i++)
		nullbuff[ i] = 0x00;

	fwrite( pv3head, 0x400, 1, fp);

	for( wavs=2; wavs<argc; wavs++){
		if ((fp2 = fopen( argv[ wavs], "rb")) == NULL){
			printf("fopen error %s %d\n", argv[ wavs], errno);
			break;
		}

		fseek( fp2, 0, 2);
		bytes = ftell( fp2);
		fprintf(stderr, "%s: %d bytes\n", argv[ wavs], bytes);
		fseek( fp2, 0, 0);
		buf = VirtualAlloc( NULL, bytes, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
		if( buf == NULL){
			printf("malloc error %d\n",GetLastError());
			break;
		}
		fread( buf, bytes, 1, fp2);
		fclose( fp2);

		datasize = enc1( fp, (short *)buf, bytes);
		if( datasize < 0x800)
			fwrite( nullbuff, 0x800-datasize, 1, fp);

		VirtualFree( buf, bytes, MEM_DECOMMIT);
	}
	for( ; wavs<33; wavs++){
		fwrite( nullbuff, 0x800, 1, fp);
	}
	fwrite( nullbuff, 0x400, 1, fp);

	fclose( fp);
	exit(0);
}

int enc1( FILE *fp, short *buf, int size)
{
	int count, loops;
	int onoff;
	unsigned char head[4] = { 0xFF, 0xFF, 0x3f, 0x00};
	int outcnt;
	short data;

	outcnt=0;
	fwrite( head, 4, 1, fp);
	outcnt++;
	for( loops=0x2c/2; loops<size/2; loops+=2){
		data = buf[ loops];
		if( data>0x1000)
			break;
	}

	onoff=1;
	count=0;
	for( ; loops<size/2; loops+=2){
		data = buf[ loops];
		if( ( data>0x1000) && ( data<0x2000)){
			count++;
			continue;
		}
		if( onoff){
			if( data>0x1000){
				count++;
				continue;
			}
			count *= 0x15C;
			fwrite( (char *)&count, 4, 1, fp);
			outcnt++;
			count = 0;
			onoff = 0;
		}else{
			if( data<=0x1000){
				count++;
				continue;
			}
			count *= 0x15C;
			fwrite( (char *)&count, 4, 1, fp);
			outcnt++;
			count = 0;
			onoff = 1;
		}
	}
	fwrite( head, 4, 1, fp);
	outcnt++;

	return outcnt*4;
}

